package livefilter

import org.bytedeco.javacpp.Loader
import org.bytedeco.javacpp.indexer.UByteRawIndexer
import org.bytedeco.opencv.global.opencv_core.CV_8U
import org.bytedeco.opencv.opencv_core.{Mat, Rect, RectVector}
import org.bytedeco.opencv.opencv_objdetect.CascadeClassifier

import java.io.File
import java.net.URL

object FaceDetection {
  val url: URL = new URL("https://raw.github.com/opencv/opencv/master/data/haarcascades/haarcascade_frontalface_alt.xml")
  val file: File = Loader.cacheResource(url)
  val classifierName: String = file.getAbsolutePath
  val cascade: CascadeClassifier = new CascadeClassifier(classifierName)

  def detectFaces(src: Array[Array[Int]]): Array[Rect] = {
    val grayFrame: Mat = new Mat()
    val width: Int = src.length
    val height: Int = if (width == 0) 0 else src(0).length
    grayFrame.create(height, width, CV_8U)

    val idx: UByteRawIndexer = grayFrame.createIndexer()

    for (x: Int <- 0 until width) {
      for (y: Int <- 0 until height) {
        idx.put(y, x, src(x)(y))
      }
    }

    val faces: RectVector = new RectVector()
    cascade.detectMultiScale(grayFrame, faces)
    return faces.get()
  }
}
